
# q-expansion of newform 576.7.g.h, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 576
weight = 7
poly_data = [4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-8, 16], 1]]

hecke_ring_character_values = [[127, [-1, 0]], [325, [1, 0]], [65, [1, 0]]]
aps_data = [[0, 0], [0, 0], [10, 0], [0, 10], [0, -31], [-1466, 0], [4766, 0], [0, -243], [0, 338], [25498, 0], [0, 1352], [-1994, 0], [-29362, 0], [0, 695], [0, 244], [-192854, 0], [0, -2531], [10918, 0], [0, 12721], [0, -17178], [288626, 0], [0, -10028], [0, -6589], [-310738, 0], [-1457086, 0], [-639158, 0], [0, 44834], [0, -37095], [-1535738, 0], [601694, 0], [0, 54048], [0, 91807], [-3810034, 0], [0, -4481], [-3274262, 0], [0, -180530], [-816794, 0], [0, 59577], [0, -257074], [-5126534, 0], [0, -75333], [-9691562, 0], [0, 368464], [-2430334, 0], [-2230646, 0], [0, 158010], [0, -126179], [0, 107512], [0, -435817], [-1595978, 0], [-8043826, 0], [0, -363196], [5051042, 0], [0, 152205], [-23455234, 0], [0, 700342], [-29427782, 0], [0, -274812], [-27622634, 0], [-8640082, 0], [0, 411019], [-44541494, 0], [0, 1578405], [0, -1614454], [1127186, 0], [34488154, 0], [0, -1299697], [-34253086, 0], [0, 1760581], [-47000858, 0], [12723134, 0], [0, 652446], [0, -358436], [-687146, 0], [0, -479277], [0, 1084064], [-101121782, 0], [34826566, 0], [68839838, 0], [45995858, 0], [0, 875143], [94207798, 0], [0, 1675668], [84021026, 0], [0, -4780394], [0, 2594045], [88092542, 0], [-37542286, 0], [115259962, 0], [0, -333772], [0, 5300259], [0, -2505272], [0, -348798], [0, -5972999], [0, 2206645], [0, -424614], [134186458, 0], [198564878, 0], [0, -6955617], [-14418842, 0], [0, -1368871], [89084794, 0], [0, -7796837], [25614062, 0], [0, -3575453], [107271554, 0], [0, 698417], [-200340706, 0], [0, 4440770], [-190305838, 0], [0, 4049240], [99111094, 0], [-37082674, 0], [0, 13080167], [0, -815418], [421292798, 0], [0, -2639135], [0, 5947734], [64384186, 0], [0, 17367059], [-283897322, 0], [277692866, 0], [-92302646, 0], [0, -8742039], [0, 621509], [352233562, 0], [-462732938, 0], [0, -14858868], [0, 15562078], [508270054, 0], [0, -4123671], [0, 9143870], [0, -6949060], [303985366, 0], [-863611282, 0], [-19789822, 0], [-564048374, 0], [0, -13045427], [-21599846, 0], [-651310258, 0], [0, -17844489], [-894865238, 0], [0, 27255542], [0, -1004067], [40544422, 0], [0, 13485094], [428994454, 0], [-220597714, 0], [0, 37041211], [0, -2081688], [-515252474, 0], [118518878, 0], [0, 4152789], [0, -16458502], [0, 17532095], [0, 19351516], [0, -31193202], [-722626498, 0], [-412016206, 0], [787666042, 0], [0, -17089669], [-1183226386, 0], [0, 15449594], [0, -22686447], [921322526, 0], [0, -7888286], [0, 57508536], [-1367900234, 0]]
